/*
 * Copyright (c) 2001-2002, Almacom Ltd (almacom.com)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS `AS IS'
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef __J2K_H
#define __J2K_H

#define J2K_MAXRLVLS 33
#define J2K_MAXBANDS (3*J2K_MAXRLVLS+1)

#define J2K_CP_CSTY_PRT 0x01
#define J2K_CP_CSTY_SOP 0x02
#define J2K_CP_CSTY_EPH 0x04
#define J2K_CCP_CSTY_PRT 0x01
#define J2K_CCP_CBLKSTY_LAZY 0x01
#define J2K_CCP_CBLKSTY_RESET 0x02
#define J2K_CCP_CBLKSTY_TERMALL 0x04
#define J2K_CCP_CBLKSTY_VSC 0x08
#define J2K_CCP_CBLKSTY_PTERM 0x10
#define J2K_CCP_CBLKSTY_SEGSYM 0x20
#define J2K_CCP_QNTSTY_NOQNT 0
#define J2K_CCP_QNTSTY_SIQNT 1
#define J2K_CCP_QNTSTY_SEQNT 2

typedef struct {
    int dx, dy;
    int prec;
    int sgnd;
    int *data;
} j2k_comp_t;

typedef struct {
    int x0, y0, x1, y1;
    int numcomps;
    j2k_comp_t *comps;
} j2k_image_t;

typedef struct {
    int expn;
    int mant;
} j2k_stepsize_t;

typedef struct {
    int csty;
    int numresolutions;
    int cblkw;
    int cblkh;
    int cblksty;
    int qmfbid;
    int qntsty;
    j2k_stepsize_t stepsizes[J2K_MAXBANDS];
    int numgbits;
    int roishift;
    int prcw[J2K_MAXRLVLS];
    int prch[J2K_MAXRLVLS];
} j2k_tccp_t;

typedef struct {
    int resno0, compno0;
    int layno1, resno1, compno1;
    int prg;
} j2k_poc_t;

typedef struct {
    int csty;
    int prg;
    int numlayers;
    int mct;
    int rates[32];
    int numpocs;
    j2k_poc_t pocs[32];
    j2k_tccp_t *tccps;
} j2k_tcp_t;

typedef struct {
    int tx0, ty0;
    int tdx, tdy;
    int tw, th;
    j2k_tcp_t *tcps;
} j2k_cp_t;

int j2k_encode(j2k_image_t *i, j2k_cp_t *cp, char *dest, int len);
int j2k_decode(char *src, int len, j2k_image_t **i, j2k_cp_t **cp);

#endif
